﻿using System;
using System.Collections.Generic;
using System.Text;

using System.IO;
using System.Net;

using Popolo.Weather;
using Popolo.Numerics;

namespace RAD_CONVERT
{
  class Program
  {

    #region 列挙型

    private const Location LOCATION = Location.Sapporo;

    private enum Location
    {
      Tokyo,
      Osaka,
      Fukuoka,
      Sapporo,
      Sendai,
      Naha
    }

    #endregion    

    static void Main(string[] args)
    {

      //dataディレクトリに必要なデータを入れて下記の計算を順に実行

      //拡張アメダス変換処理
      //convertAMEDAS();

      //PP.WEBデータ（気象庁）取得処理//Unryo.csvを作る//気象庁のwebサイト構成が変わる可能性あり
      //getWeb();

      //00.雲量の確認
      //checkCloud();

      //01.大気透過率の計算
      //makeAtmosTrans();

      //02.トレンドの分離
      //removeTrend();

      //03.日別平均データの作成
      //makeDailyAverageData();

      //04.年周期のフーリエ級数を計算
      //Rを使用

      //05.時刻別平均データの作成
      //makeHourlyAverageData();

      //06.日周期のフーリエ級数を計算
      //Rを使用

      //07.振幅変調係数の計算
      //makeSwingCoefficient();

      //08.ランダム項の分離
      //removeRandom();

      //09.雲量でデータを分ける
      //selectCC();

      //10.大気透過率のARモデル入力データを作成
      //makeARData_CC();
      //makeARModel_CC();←重すぎ

      //11.ARモデル年周期標準偏差を求める
      //Rを使用

      //12.VARモデル入力データを作成
      //makeVARInputs();

      //13.VARモデルの係数を計算
      //Rを使用

      //14.テスト
      //test();

      //15.サンプルデータ作成
      //makeSampleData();
    }

    #region 拡張アメダスデータ変換処理

    private static void convertAMEDAS()
    {
      //0:00のデータは0:00~1:00のデータを意味することに注意する
      string path = Directory.GetCurrentDirectory();
      DateTime dTime = new DateTime(1981, 1, 1, 0, 0, 0);

      using (StreamWriter sWriterH = new StreamWriter("outH.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamWriter sWriterD = new StreamWriter("outD.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamWriter sWriterM = new StreamWriter("outM.csv", false, Encoding.GetEncoding("Shift_JIS")))
      {
        for (int i = 1981; i <= 2000; i++)
        {
          using (StreamReader sReaderT = new StreamReader(path + "\\data\\" + i + "t.csv", Encoding.GetEncoding("Shift_JIS")))
          using (StreamReader sReaderX = new StreamReader(path + "\\data\\" + i + "x.csv", Encoding.GetEncoding("Shift_JIS")))
          using (StreamReader sReaderI = new StreamReader(path + "\\data\\" + i + "i.csv", Encoding.GetEncoding("Shift_JIS")))
          {
            sReaderT.ReadLine();//タイトル行
            sReaderX.ReadLine();//タイトル行
            sReaderI.ReadLine();//タイトル行
            string buff;
            string[] b1, b2, b3;
            int lastM = 12;
            double tSumM, xSumM, iSumM, iMaxM;
            int days = 0;
            tSumM = xSumM = iSumM = iMaxM = 0;
            while ((buff = sReaderT.ReadLine()) != null)
            {
              sWriterD.Write(dTime.ToShortDateString() + ", ");
              if (dTime.Month != lastM)
              {
                days = 0;
                sWriterM.Write(dTime.Year + "/" + dTime.Month + ", ");
                tSumM = xSumM = iSumM = iMaxM = 0;
                lastM = dTime.Month;
              }
              double tSum, xSum, iSum, iMax;
              tSum = xSum = iSum = iMax = 0;
              b1 = buff.Split(',');
              b2 = sReaderX.ReadLine().Split(',');
              b3 = sReaderI.ReadLine().Split(',');
              for (int j = 1; j <= 24; j++)
              {
                tSum += double.Parse(b1[j]);
                xSum += double.Parse(b2[j]);
                iSum += double.Parse(b3[j]);
                iMax = Math.Max(iMax, double.Parse(b3[j]));
                sWriterH.Write(dTime.ToShortDateString() + "," + dTime.ToShortTimeString());
                sWriterH.Write(", " + b1[j]);
                sWriterH.Write(", " + b2[j]);
                sWriterH.Write(", " + b3[j]);
                sWriterH.WriteLine();
                dTime = dTime.AddHours(1);
              }
              days++;
              sWriterD.WriteLine
                ((tSum / 24d).ToString("F3") + ", " + (xSum / 24d).ToString("F3") + ", " + iMax.ToString("F3"));
              tSumM += tSum;
              xSumM += xSum;
              iSumM += iSum;
              iMaxM += iMax;
              if (dTime.Month != lastM)
              {
                int hours = 24 * days;
                sWriterM.WriteLine
                ((tSumM / hours).ToString("F3") + ", " + (xSumM / hours).ToString("F3") + ", " + (iMaxM / days).ToString("F3"));
              }
            }
          }
        }
      }
    }

    #endregion

    #region 雲量処理

    private static void selectCC()
    {
      using (StreamWriter sWriter1 = new StreamWriter("Unryo10.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamWriter sWriter2 = new StreamWriter("Unryo00.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamWriter sWriter3 = new StreamWriter("Unryo00And10.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamReader sReader1 = new StreamReader("UnryoDay.csv", Encoding.GetEncoding("Shift_JIS")))
      using (StreamReader sReader2 = new StreamReader("random.csv", Encoding.GetEncoding("Shift_JIS")))
      {
        sReader2.ReadLine();

        //雲量データを1年分飛ばす
        for (int i = 0; i < 365; i++) sReader1.ReadLine();

        DateTime dt = new DateTime(1982, 1, 1, 0, 0, 0);
        string sb1;
        while ((sb1 = sReader1.ReadLine()) != null)
        {
          int unryo = int.Parse(sb1.Split(',')[1]);

          //うるう年2/29は無視
          if (!(dt.Month == 2 && dt.Day == 29))
          {
            if (unryo == 10)
            {
              sWriter1.Write(dt.ToShortDateString());
              sWriter3.Write(dt.ToShortDateString());
            }
            else if (unryo == 0)
            {
              sWriter2.Write(dt.ToShortDateString());
              sWriter3.Write(dt.ToShortDateString());
            }

            for (int i = 0; i < 24; i++)
            {
              sb1 = sReader2.ReadLine();
              string val = sb1.Split(',')[10];
              if (double.Parse(val) == 0) val = " ";
              if (unryo == 10)
              {
                sWriter1.Write("," + val);
                sWriter3.Write("," + val);
              }
              else if (unryo == 0)
              {
                sWriter2.Write("," + val);
                sWriter3.Write("," + val);
              }
            }

            if (unryo == 10)
            {
              sWriter1.WriteLine();
              sWriter3.WriteLine();
            }
            else if (unryo == 0)
            {
              sWriter2.WriteLine();
              sWriter3.WriteLine();
            }
          }
          dt = dt.AddDays(1);
          if (1999 < dt.Year) break;
        }

      }
    }

    private static void getWeb()
    {
      //2014.11.09時点の気象庁データ、雲量読み込みプログラム
      DateTime dt = new DateTime(1981, 1, 1);

      using (StreamWriter sWriter = new StreamWriter("Unryo.csv", false, Encoding.GetEncoding("Shift_JIS")))
      {
        for (int j = 0; j < 365 * 20 + 5; j++)
        {
          WebClient client = new WebClient();
          string str;
          switch (LOCATION)
          {
            case Location.Osaka:
              str = client.DownloadString("http://www.data.jma.go.jp/obd/stats/etrn/view/hourly_s1.php?prec_no=62&block_no=47772&year=" + dt.Year + "&month=" + dt.Month + "&day=" + dt.Day + "&view=");
              break;
            case Location.Naha:
              str = client.DownloadString("http://www.data.jma.go.jp/obd/stats/etrn/view/hourly_s1.php?prec_no=91&block_no=47936&year=" + dt.Year + "&month=" + dt.Month + "&day=" + dt.Day + "&view=");
              break;
            case Location.Sapporo:
              str = client.DownloadString("http://www.data.jma.go.jp/obd/stats/etrn/view/hourly_s1.php?prec_no=14&block_no=47412&year=" + dt.Year + "&month=" + dt.Month + "&day=" + dt.Day + "&view=");
              break;
            case Location.Sendai:
              str = client.DownloadString("http://www.data.jma.go.jp/obd/stats/etrn/view/hourly_s1.php?prec_no=34&block_no=47590&year=" + dt.Year + "&month=" + dt.Month + "&day=" + dt.Day + "&view=");
              break;
            case Location.Fukuoka:
              str = client.DownloadString("http://www.data.jma.go.jp/obd/stats/etrn/view/hourly_s1.php?prec_no=82&block_no=47807&year=" + dt.Year + "&month=" + dt.Month + "&day=" + dt.Day + "&view=");
              break;
            default:
              str = client.DownloadString("http://www.data.jma.go.jp/obd/stats/etrn/view/hourly_s1.php?prec_no=44&block_no=47662&year=" + dt.Year + "&month=" + dt.Month + "&day=" + dt.Day + "&view=");
              break;
          }
          //該当部分抜き出し
          string[] ss = str.Split('\n');
          for (int i = 0; i < 7; i++)
          {
            string[] ddd = ss[128 + i * 3].Split(new string[] { "</td><td class=\"data_0_0\">" }, StringSplitOptions.None);
            string sbb = ddd[ddd.Length - 2].TrimEnd(new char[] { '+', '-' });
            sWriter.WriteLine(dt.ToShortDateString() + "," + (1 + i) * 3 + "," + sbb);
          }
          dt = dt.AddDays(1);
          Console.WriteLine(dt.ToShortDateString());
        }
      }
    }

    private static void checkCloud()
    {
      using (StreamWriter sWriter1 = new StreamWriter("UnryoRate.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamWriter sWriter2 = new StreamWriter("UnryoAll.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamReader sReader = new StreamReader("Unryo.csv", Encoding.GetEncoding("Shift_JIS")))
      {
        DateTime dt = new DateTime(1981, 1, 1, 0, 0, 0);
        int[] cc10 = new int[365];
        int[] cc00 = new int[365];
        int day = -1;

        string sb;
        while ((sb = sReader.ReadLine()) != null)
        {
          string[] sb2 = sb.Split(',');

          //曇天日率を作成
          if (!(dt.Month == 2 && dt.Day == 29))
          {
            if (dt.Hour == 0)
            {
              day++;
              if (day == 365) day = 0;
            }
            if (int.Parse(sb2[2]) == 10) cc10[day]++;
            else cc00[day]++;
          }

          //全時間の雲量データを作成
          if (sb2[1] == "3")
          {
            sWriter2.WriteLine(dt.ToShortDateString() + ", 0, " + sb2[2]);
            sWriter2.WriteLine(dt.ToShortDateString() + ", 1, " + sb2[2]);
            dt = dt.AddHours(2);
          }
          sWriter2.WriteLine(dt.ToShortDateString() + ", " + dt.Hour + ", " + sb2[2]);
          sWriter2.WriteLine(dt.ToShortDateString() + ", " + (dt.Hour + 1) + ", " + sb2[2]);
          sWriter2.WriteLine(dt.ToShortDateString() + ", " + (dt.Hour + 2) + ", " + sb2[2]);
          dt = dt.AddHours(3);
          if (sb2[1] == "21")
          {
            sWriter2.WriteLine(dt.ToShortDateString() + ", 23, " + sb2[2]);
            dt = dt.AddHours(1);
          }
        }

        for (int i = 0; i < cc10.Length; i++)
          sWriter1.WriteLine((i + 1) + ", " + cc10[i] + ", " + cc00[i] + ", " + ((double)cc10[i] / (cc10[i] + cc00[i])));
      }

      //雲量終日10, 9未満の切り分け
      using (StreamReader sReader = new StreamReader("Unryo.csv", Encoding.GetEncoding("Shift_JIS")))
      using (StreamWriter sWriter = new StreamWriter("UnryoDay.csv", false, Encoding.GetEncoding("Shift_JIS")))
      {
        string sb1;
        int cc = -1;
        bool same = false;
        int maxCC = 0;
        const int MAXCC = 9;
        DateTime dt = new DateTime(1981, 1, 1, 0, 0, 0);
        while ((sb1 = sReader.ReadLine()) != null)
        {
          string[] buff2 = sb1.Split(',');
          if (buff2[1] == "3")
          {
            same = true;
            cc = maxCC = int.Parse(buff2[2]);
          }
          else
          {
            if (cc != int.Parse(buff2[2])) same = false;
            maxCC = Math.Max(maxCC, int.Parse(buff2[2]));

            if (buff2[1] == "21")
            {
              if (same && cc == 10) sWriter.WriteLine(dt.ToShortDateString() + ",10");
              else if (maxCC <= MAXCC) sWriter.WriteLine(dt.ToShortDateString() + ",0");
              else sWriter.WriteLine(dt.ToShortDateString() + ",5");

              cc = -1;
              same = false;
              maxCC = 0;
              dt = dt.AddDays(1);
            }
          }
        }
      }

      //推移確率の計算
      using (StreamReader sReader = new StreamReader("Unryo.csv", Encoding.GetEncoding("Shift_JIS")))
      using (StreamWriter sWriter = new StreamWriter("UnryoTrans.csv", false, Encoding.GetEncoding("Shift_JIS")))
      {
        int[] FToF = new int[365];  //晴れ→晴れ
        int[] FToC = new int[365];  //晴れ→曇り
        int[] CToF = new int[365];  //曇り→晴れ
        int[] CToC = new int[365];  //曇り→曇り

        bool lastFair = false;
        string sb1;
        int day = 0;
        DateTime dt = new DateTime(1981, 1, 1, 0, 0, 0);
        while ((sb1 = sReader.ReadLine()) != null)
        {
          string[] buff2 = sb1.Split(',');

          //うるう年は飛ばす
          if (!(dt.Month == 2 && dt.Day == 29))
          {
            bool curFair = (int.Parse(buff2[2]) != 10);
            if (buff2[1] != "3")
            {
              if (lastFair && curFair) FToF[day]++;
              else if (lastFair && !curFair) FToC[day]++;
              else if (!lastFair && curFair) CToF[day]++;
              else CToC[day]++;
            }

            if (buff2[1] == "21")
            {
              day++;
              if (day == 365) day = 0;
            }
            lastFair = curFair;
          }
          if (buff2[1] == "21") dt = dt.AddDays(1);
        }

        //書き出し
        sWriter.WriteLine("日付,晴れ→晴れ,晴れ→曇り,曇り→晴れ,曇り→曇り,(晴れ→晴れ)確率,(曇り→曇り)確率,不変分布（晴れ）,不変分布（曇り）");
        for (int i = 0; i < FToF.Length; i++)
        {
          double ff = (double)FToF[i] / (FToF[i] + FToC[i]);
          double cc = (double)CToC[i] / (CToF[i] + CToC[i]);
          double idisF = (1 - cc) / (2 - ff - cc);
          double idisC = (1 - ff) / (2 - ff - cc);
          sWriter.WriteLine((i + 1) + "," + FToF[i] + "," + FToC[i] + "," + CToF[i] + "," + CToC[i] + "," + ff + "," + cc + "," + idisF + "," + idisC);
        }
      }

    }

    #endregion

    #region 大気透過率の計算

    private static void makeAtmosTrans()
    {
      //拡張アメダスは毎正時前後30分のデータ
      DateTime dTime = new DateTime(1981, 1, 1, 1, 0, 0);
      Sun sun;
      switch (LOCATION)
      {
        case Location.Osaka:
          sun = new Sun(34, 40.7, 0, 135, 31.3, 0, 135, 0, 0);
          break;
        case Location.Sapporo:
          sun = new Sun(43, 3.5, 0, 141, 19.9, 0, 135, 0, 0);
          break;
        case Location.Sendai:
          sun = new Sun(38, 15.5, 0, 140, 54.0, 0, 135, 0, 0);
          break;
        case Location.Fukuoka:
          sun = new Sun(33, 34.8, 0, 130, 22.6, 0, 135, 0, 0);
          break;
        case Location.Naha:
          sun = new Sun(26, 12.2, 0, 127, 41.3, 0, 135, 0, 0);
          break;
        default:
          sun = new Sun(35, 41.2, 0, 139, 45.9, 0, 135, 0, 0);
          break;
      }
      sun.Update(dTime);
      DateTime sunset = sun.GetSunSetTime();
      DateTime sunrise = sun.GetSunRiseTime();

      using (StreamWriter sWriter1 = new StreamWriter("outH_Rad.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamReader sReader1 = new StreamReader("outH.csv", Encoding.GetEncoding("Shift_JIS")))
      using (StreamReader sReader2 = new StreamReader("unryoAll.csv", Encoding.GetEncoding("Shift_JIS")))
      {
        sWriter1.WriteLine("日付,時刻,乾球温度[C], 絶対湿度[g/kg], 水平面全天日射[W/m2],大気圏外水平面日射[W/m2], sin(太陽高度)[-], 計算成功[-], 大気透過率[-], 雲量");

        string buff;
        DateTime bDt = new DateTime(1981, 1, 1, 1, 0, 0);
        List<double> time = new List<double>();
        List<double> val = new List<double>();
        while ((buff = sReader1.ReadLine()) != null)
        {
          string unryo = sReader2.ReadLine().Split(',')[2];

          //うるう年 2月29日は無視
          if (!(dTime.Month == 2 && dTime.Day == 29))
          {
            //日の出・日没時刻更新
            if (sunset.Day != dTime.Day)
            {
              sun.Update(dTime);
              sunset = sun.GetSunSetTime();
              sunrise = sun.GetSunRiseTime();
            }

            string[] bf2 = buff.Split(',');
            double rad = (double.Parse(bf2[4]) / 0.0036);
            sWriter1.Write(bf2[0] + "," + (dTime.Hour) + "," + bf2[2] + "," + bf2[3] + "," + rad.ToString("F2") + ",");

            //日中ならば正規化//日没日の出から前後30分は誤差大きいためカット
            if ((sunrise.AddMinutes(60) < dTime) && (dTime < sunset.AddMinutes(-60)))
            {
              sun.Update(dTime);
              //大気圏外日射
              double sinH = Math.Sin(sun.Altitude);
              double exr = sun.GetExtraterrestrialRadiation();
              //大気透過率[-]
              bool suc;
              double atmTrans = getATMTrans(sinH, exr, rad, out suc);
              //書き出し
              sWriter1.WriteLine((exr * sinH) + "," + Math.Sin(sun.Altitude) + "," + suc.ToString() + "," + atmTrans + ", " + unryo);
            }
            else sWriter1.WriteLine("0,0,0,0," + unryo);
          }

          //1時間更新
          dTime = dTime.AddHours(1);
        }
      }
    }

    private static double getATMTrans
      (double sinH, double exRadiation, double globalHorizontalRadiation, out bool success)
    {
      double dn, dff;
      success = false;

      //大気透過率=0で観測値を上回る場合
      getDirectAndDiffuseRadiation(0, sinH, exRadiation, out dn, out dff);
      if (globalHorizontalRadiation < dn * sinH + dff) return 0;

      //大気透過率=1で観測値を下回る場合
      getDirectAndDiffuseRadiation(1, sinH, exRadiation, out dn, out dff);
      if (dn * sinH + dff < globalHorizontalRadiation) return 1;

      //大気透過率を二分法で収束計算
      //誤差関数を定義
      success = true;
      Roots.ErrorFunction eFnc = delegate(double atmTrans)
      {
        getDirectAndDiffuseRadiation(atmTrans, sinH, exRadiation, out dn, out dff);
        return globalHorizontalRadiation - (dn * sinH + dff);
      };
      return Roots.Bisection(0, 1, 0.0001, eFnc);
    }

    /// <summary>渡辺の手法で法線面直達日射[W/m2]と水平面天空日射[W/m2]を計算する</summary>
    /// <param name="aTransmissivity">大気透過率[-]</param>
    /// <param name="sinAltitude">太陽高度の正弦</param>
    /// <param name="exRadiation">大気圏外日射[W/m2]</param>
    /// <param name="directNormalRadiation">法線面直達日射[W/m2]</param>
    /// <param name="diffuseHorizontalRadiation">水平面天空日射[W/m2]</param>
    private static void getDirectAndDiffuseRadiation
      (double aTransmissivity, double sinAltitude, double exRadiation,
      out double directNormalRadiation, out double diffuseHorizontalRadiation)
    {
      double ps = Math.Pow(aTransmissivity, 1 / sinAltitude);
      directNormalRadiation = exRadiation * ps;
      
      //渡辺の式
      double q = (0.8672 + 0.7505 * sinAltitude) *
        Math.Pow(aTransmissivity, 0.421 / sinAltitude) *
        Math.Pow(1d - Math.Pow(aTransmissivity, 1 / sinAltitude), 2.277);
      diffuseHorizontalRadiation = exRadiation * sinAltitude * (q / (1 + q));
    }

    #endregion
    
    #region トレンドの分離

    /// <summary>トレンドを分離する</summary>
    private static void removeTrend()
    {
      using (StreamWriter sWriter = new StreamWriter("outH_Rad_Trend.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamReader sReader = new StreamReader("outH_Rad.csv", Encoding.GetEncoding("Shift_JIS")))
      {
        sReader.ReadLine();
        sWriter.WriteLine("日付,時刻,乾球温度[C],絶対湿度[g/kg],大気透過率[-],移動平均乾球温度[C],移動平均絶対湿度[g/kg],移動平均大気透過率[-],補正後乾球温度[C],補正後絶対湿度[g/kg],補正後大気透過率[-],雲量");

        DateTime dt = new DateTime(1981, 1, 1, 0, 0, 0);
        DateTime dtn = dt.AddHours(-8760 / 2 + 1);
        string bf1;
        double sDBT = 0;
        double sHRT = 0;
        double sRAD = 0;
        double sSOL = 0;
        double[] dbt = new double[8761];
        double[] hrt = new double[8761];
        double[] rad = new double[8761];
        double[] sol = new double[8761];
        int[] cc = new int[8761];
        int hour = 0;
        int hourNow = 4381;
        while ((bf1 = sReader.ReadLine()) != null)
        {
          string[] bf2 = bf1.Split(',');
          int hourm1, hourp1;
          if (hour == 0) hourm1 = 8760;
          else hourm1 = hour - 1;
          if (hour == 8760) hourp1 = 0;
          else hourp1 = hour + 1;

          sDBT -= dbt[hour] / 2d;
          sHRT -= hrt[hour] / 2d;
          sRAD -= rad[hour] / 2d;
          sSOL -= sol[hour] / 2d;
          sDBT -= dbt[hourp1] / 2d;
          sHRT -= hrt[hourp1] / 2d;
          sRAD -= rad[hourp1] / 2d;
          sSOL -= sol[hourp1] / 2d;
          dbt[hour] = double.Parse(bf2[2]);
          hrt[hour] = double.Parse(bf2[3]);
          rad[hour] = double.Parse(bf2[8]);
          cc[hour] = int.Parse(bf2[9]);
          if (double.Parse(bf2[8]) == 0) sol[hour] = 0;
          else sol[hour] = 1;
          sDBT += dbt[hour] / 2d;
          sHRT += hrt[hour] / 2d;
          sRAD += rad[hour] / 2d;
          sSOL += sol[hour] / 2d;
          sDBT += dbt[hourm1] / 2d;
          sHRT += hrt[hourm1] / 2d;
          sRAD += rad[hourm1] / 2d;
          sSOL += sol[hourm1] / 2d;

          if (1981 < dtn.Year && dtn.Year < 2000)
          {
            double mDBT = sDBT / 8760d;
            double mHRT = sHRT / 8760d;
            double mRAD = sRAD / sSOL;
            double modRAD = rad[hourNow] - mRAD;
            if (rad[hourNow] == 0) mRAD = modRAD = 0;
            sWriter.WriteLine(dtn.ToShortDateString() + "," + dtn.Hour + "," +
              dbt[hourNow] + "," + hrt[hourNow] + "," + rad[hourNow] + "," +
              mDBT + "," + mHRT + "," + mRAD + "," +
              (dbt[hourNow] - mDBT) + "," +
              (hrt[hourNow] - mHRT) + "," +
              modRAD + "," +cc[hourNow]);
          }

          dtn = dtn.AddHours(1);
          if (dtn.Month == 2 && dtn.Day == 29) dtn = dtn.AddDays(1);//うるう年 2月29日は無視
          dt = dt.AddHours(1);
          if (dt.Month == 2 && dt.Day == 29) dt = dt.AddDays(1);//うるう年 2月29日は無視

          hour++;
          hourNow++;
          if (hour == 8761) hour = 0;
          if (hourNow == 8761) hourNow = 0;
        }
      }
    }

    #endregion

    #region 日周期・年周期成分の作成

    private static void makeDailyAverageData()
    {
      using (StreamWriter sWriter1 = new StreamWriter("hourlyData.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamWriter sWriter2 = new StreamWriter("dailyData.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamReader sReader1 = new StreamReader("outH_Rad_Trend.csv", Encoding.GetEncoding("Shift_JIS")))
      {
        sReader1.ReadLine();

        sWriter1.WriteLine("日付,時刻,乾球温度[C],絶対湿度[g/kg],日平均大気透過率[-]");
        sWriter2.WriteLine("日付,乾球温度[C],絶対湿度[g/kg],日平均大気透過率[-]");

        DateTime dt = new DateTime(1982, 1, 1, 0, 0, 0);
        string buff;
        double sDBT = 0;
        double sHRT = 0;
        double sRAD = 0;
        int nRAD = 0;
        while ((buff = sReader1.ReadLine()) != null)
        {
          string[] bf2 = buff.Split(',');

          //時刻別データ書き出し
          sWriter1.WriteLine
            (dt.ToShortDateString() + "," + dt.Hour + "," +
            bf2[8] + "," + bf2[9] + "," + bf2[10]);

          sDBT += double.Parse(bf2[8]);
          sHRT += double.Parse(bf2[9]);
          sRAD += double.Parse(bf2[10]);
          if (double.Parse(bf2[10]) != 0) nRAD++;

          //日付が変わる前に日平均データ書き出し
          if (dt.Day != dt.AddHours(1).Day)
          {
            sWriter2.WriteLine
              (dt.ToShortDateString() + "," +
               (sDBT / 24d) + "," + (sHRT / 24d) + "," + (sRAD / nRAD).ToString("F2"));
            sDBT = sHRT = sRAD = nRAD = 0;
          }

          dt = dt.AddHours(1);
          //うるう年対応
          if (dt.Month == 2 && dt.Day == 29) dt = dt.AddDays(1);
        }
      }

      //時刻別平均値データの作成
      using (StreamWriter sWriter = new StreamWriter("hourlyDataAVE.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamReader sReader = new StreamReader("hourlyData.csv", Encoding.GetEncoding("Shift_JIS")))
      {
        sReader.ReadLine();
        DateTime dt = new DateTime(1981, 1, 1, 0, 0, 0);
        string buff;
        Dictionary<string, double> hdDBT = new Dictionary<string, double>();
        Dictionary<string, double> hdHRT = new Dictionary<string, double>();
        Dictionary<string, double> hdRAD = new Dictionary<string, double>();
        for (int i = 0; i < 8760; i++)
        {
          hdDBT[dt.ToString("MMddHH")] = 0;
          hdHRT[dt.ToString("MMddHH")] = 0;
          hdRAD[dt.ToString("MMddHH")] = 0;
          dt = dt.AddHours(1);
        }
        dt = new DateTime(1982, 1, 1, 0, 0, 0);
        while ((buff = sReader.ReadLine()) != null)
        {
          //うるう年対応
          if (!(dt.Month == 2 && dt.Day == 29))
          {
            string[] buff2 = buff.Split(',');
            hdDBT[dt.ToString("MMddHH")] += double.Parse(buff2[2]);
            hdHRT[dt.ToString("MMddHH")] += double.Parse(buff2[3]);
            hdRAD[dt.ToString("MMddHH")] += double.Parse(buff2[4]);
          }
          dt = dt.AddHours(1);
        }

        sWriter.WriteLine("月日,時刻,乾球温度[C],絶対湿度[g/kg],大気透過率[-]");
        dt = new DateTime(1999, 1, 1, 0, 0, 0);
        for (int i = 0; i < 8760; i++)
        {
          string str = dt.ToString("MMddHH");
          sWriter.Write(dt.ToString("MM/dd") + ", " + dt.Hour + ",");
          sWriter.Write((hdDBT[str] / 18d).ToString() + ", ");
          sWriter.Write((hdHRT[str] / 18d).ToString() + ", ");
          sWriter.WriteLine((hdRAD[str] / 18d).ToString());
          dt = dt.AddHours(1);
        }
      }

      //日別平均データの作成
      using (StreamWriter sWriter = new StreamWriter("dailyDataAVE.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamReader sReader = new StreamReader("hourlyDataAVE.csv", Encoding.GetEncoding("Shift_JIS")))
      using (StreamReader sReader2 = new StreamReader("UnryoRate.csv", Encoding.GetEncoding("Shift_JIS")))
      {
        sReader.ReadLine();

        string buff;
        int hour = 0;
        int radN = 0;
        double aveDBT = 0;
        double aveHRT = 0;
        double aveRAD = 0;
        while ((buff = sReader.ReadLine()) != null)
        {
          string[] buff2 = buff.Split(',');
          aveDBT += double.Parse(buff2[2]);
          aveHRT += double.Parse(buff2[3]);
          aveRAD += double.Parse(buff2[4]);
          if (double.Parse(buff2[4]) != 0) radN++;

          hour++;
          if (hour == 24)
          {
            string cc = sReader2.ReadLine().Split(',')[3];
            sWriter.Write(buff2[0] + ",");
            sWriter.Write((aveDBT / 24) + ",");
            sWriter.Write((aveHRT / 24) + ",");
            sWriter.Write((aveRAD / radN) + ",");
            sWriter.WriteLine(cc);
            aveDBT = aveHRT = aveRAD = radN = 0;
            hour = 0;
          }
        }
      }
    }

    private static void makeHourlyAverageData()
    {
      double[] hDBT365, hHRT365, hRAD365, hCC365, hDBT, hHRT, hRAD, hCC;
      makeDailyDataWithFFT(out hDBT365, out hHRT365, out hRAD365, out hCC365);
      interpolate365Data(hDBT365, hHRT365, hRAD365, hCC365, out hDBT, out hHRT, out hRAD, out hCC);

      using (StreamWriter sWriter1 = new StreamWriter("hourlyDataAVE_RemoveSY.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamWriter sWriter2 = new StreamWriter("hourlyDataAVE_RemoveSY_24.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamReader sReader1 = new StreamReader("hourlyDataAVE.csv", Encoding.GetEncoding("Shift_JIS")))
      using (StreamReader sReader2 = new StreamReader("hourlyData.csv", Encoding.GetEncoding("Shift_JIS")))
      using (StreamReader sReader3 = new StreamReader("UnryoDay.csv", Encoding.GetEncoding("Shift_JIS")))
      {
        sReader1.ReadLine();
        sWriter1.WriteLine("日付,時刻,乾球温度,絶対湿度,大気透過率,乾球温度-S,絶対湿度-S,大気透過率-S");

        string buff1;
        int hour = 0;
        double[] DBT24 = new double[24];
        double[] HRT24 = new double[24];
        double[] RAD24 = new double[24];
        int[] nRAD24 = new int[24];
        while ((buff1 = sReader1.ReadLine()) != null)
        {
          sWriter1.Write(buff1 + ",");
          string[] buff2 = buff1.Split(',');
          double dDBT = double.Parse(buff2[2]);
          double dHRT = double.Parse(buff2[3]);
          double dRAD = double.Parse(buff2[4]);

          DBT24[hour % 24] += (dDBT - hDBT[hour]);
          HRT24[hour % 24] += (dHRT - hHRT[hour]);
          if (dRAD != 0)
          {
            RAD24[hour % 24] += (dRAD - hRAD[hour]);
            nRAD24[hour % 24]++;
          }

          sWriter1.Write((dDBT - hDBT[hour]) + ",");
          sWriter1.Write((dHRT - hHRT[hour]) + ",");
          if (dRAD == 0) sWriter1.Write("");
          else sWriter1.Write((dRAD - hRAD[hour]));
          sWriter1.WriteLine();
          hour++;
        }
        
        //雲量によって切り分け
        hour = 0;
        double[] UN10_24 = new double[24];
        double[] UN00_24 = new double[24];
        int[] nUN10_24 = new int[24];
        int[] nUN00_24 = new int[24];
        for (int i = 0; i < 365; i++) sReader3.ReadLine();//雲量データを1年飛ばす
        DateTime dt = new DateTime(1982, 1, 1, 0, 0, 0);
        int unryo = -1;
        sReader2.ReadLine();
        while ((buff1 = sReader2.ReadLine()) != null)
        {
          if (dt.Hour == 0) unryo = int.Parse(sReader3.ReadLine().Split(',')[1]);

          string[] buff2 = buff1.Split(',');
          double rd = double.Parse(buff2[4]);
          if (rd != 0)
          {
            rd -= hRAD[hour];
            if (unryo == 10)
            {
              UN10_24[dt.Hour] += rd;
              nUN10_24[dt.Hour]++;
            }
            else if(unryo == 0)
            {
              UN00_24[dt.Hour] += rd;
              nUN00_24[dt.Hour]++;
            }
          }

          hour++;
          if (hour == 8760) hour = 0;
          dt = dt.AddHours(1);
          if (dt.Month == 2 && dt.Day == 29)
          {
            dt = dt.AddDays(1);//うるう年対応
            sReader3.ReadLine();
          }
        }

        //平均大気透過率
        for (int i = 0; i < 24; i++)
        {
          if (nRAD24[i] != 0) RAD24[i] /= nRAD24[i];
          if (nUN10_24[i] != 0) UN10_24[i] /= nUN10_24[i];
          if (nUN00_24[i] != 0) UN00_24[i] /= nUN00_24[i];
        }

        //24h書き出し
        sWriter2.WriteLine("時刻,乾球温度,絶対湿度,大気透過率,大気透過率（雲量10）,大気透過率（雲量9未満）");
        for (int i = 0; i < 24; i++)
        {
          sWriter2.Write(i + ",");
          sWriter2.Write((DBT24[i] / 365) + ",");
          sWriter2.Write((HRT24[i] / 365) + ",");
          sWriter2.Write(RAD24[i] + ",");
          sWriter2.Write(UN10_24[i] + ",");
          sWriter2.WriteLine(UN00_24[i]);
        }
      }
    }

    private static void makeDailyDataWithFFT
      (out double[] dbt, out double[] hrt, out double[] rad, out double[] cc)
    {
      //年周期フーリエ級数係数
      double[] anDBT, bnDBT, anHRT, bnHRT, anRAD, bnRAD, anFF, bnFF, anCC, bnCC;
      switch (LOCATION)
      {
        case Location.Osaka:
          anDBT = new double[] { 0.0182, -10.0338, -0.4355, 0.5670, -0.2672 };
          bnDBT = new double[] { 0.0000, 4.7647, -0.4687, 0.5458, -0.2661 };
          anHRT = new double[] { 0.0118, -5.6411, 0.5613, 0.2074, -0.0769 };
          bnHRT = new double[] { 0.0000, 2.9840, -1.2835, 0.4779, -0.1225 };
          anRAD = new double[] { 0.0120, 0.0893, 0.0061, 0.0107, -0.0155 };
          bnRAD = new double[] { 0.0000, 0.0160, 0.0155, 0.0103, -0.0127 };
          anFF = new double[] { 0.8038, 0.0436, -0.0113, 0.0130, -0.0134 };
          bnFF = new double[] { 0.0000, 0.0167, 0.0123, 0.0060, -0.0088 };
          anCC = new double[] { 0.7647, -0.0575, -0.0394, -0.0255, 0.0272 };
          bnCC = new double[] { 0.0000, -0.0138, 0.0348, -0.0238, 0.0087 };
          break;
        case Location.Sapporo:
          anDBT = new double[] { 0.0136, -11.5445, -0.7236, 0.5122, -0.2416 };
          bnDBT = new double[] { 0.0000, 5.3594, -0.4775, 0.5302, -0.2817 };
          anHRT = new double[] { 0.0070, -4.3882, 0.5599, 0.2440, -0.2687 };
          bnHRT = new double[] { 0.0000, 2.3810, -1.3414, 0.4957, -0.1201 };
          anRAD = new double[] { 0.0146, 0.0967, -0.0072, -0.0008, 0.0109 };
          bnRAD = new double[] { 0.0000, 0.0038, 0.0028, -0.0116, 0.0003 };
          anFF = new double[] { 0.7343, -0.0526, -0.0393, 0.0037, 0.0138 };
          bnFF = new double[] { 0.0000, 0.0256, 0.0114, -0.0118, -0.0041 };
          anCC = new double[] { 0.7694, -0.0497, 0.0083, 0.0141, 0.0022 };
          bnCC = new double[] { 0.0000, -0.0067, -0.0044, 0.0022, 0.0001 };
          break;
        case Location.Sendai:
          anDBT = new double[] { 0.0148, -9.6166, -0.6660, 0.6683, -0.4191 };
          bnDBT = new double[] { 0.0000, 5.0537, -0.4351, 0.4763, -0.1837 };
          anHRT = new double[] { 0.0088, -5.1868, 0.4997, 0.4007, -0.3268 };
          bnHRT = new double[] { 0.0000, 2.9537, -1.4521, 0.4731, -0.0609 };
          anRAD = new double[] { 0.0169, 0.1609, -0.0026, 0.0128, -0.0174 };
          bnRAD = new double[] { 0.0000, -0.0168, 0.0156, 0.0043, -0.0167 };
          anFF = new double[] { 0.7986, 0.0491, -0.0205, 0.0078, -0.0144 };
          bnFF = new double[] { 0.0000, -0.0169, 0.0182, -0.0022, -0.0056 };
          anCC = new double[] { 0.7758, -0.1076, -0.0254, -0.0058, 0.0141 };
          bnCC = new double[] { 0.0000, 0.0066, 0.0007, -0.0130, 0.0040 };
          break;
          case Location.Fukuoka:
          anDBT = new double[] { 0.0115, -9.3956, -0.3928, 0.3107, -0.2057 };
          bnDBT = new double[] { 0.0000, 4.4420, -0.5665, 0.5255, -0.3072 };
          anHRT = new double[] { 0.0105, -6.0004, 0.5700, 0.1399, -0.0428 };
          bnHRT = new double[] { 0.0000, 2.8796, -1.3209, 0.5317, -0.0812 };
          anRAD = new double[] { 0.0071, 0.0655, -0.0209, 0.0046, -0.0212 };
          bnRAD = new double[] { 0.0000, 0.0317, 0.0183, 0.0068, -0.0205 };
          anFF = new double[] { 0.8090, 0.0109, -0.0331, 0.0088, -0.0146 };
          bnFF = new double[] { 0.0000, 0.0223, 0.0094, 0.0152, -0.0122 };
          anCC = new double[] { 0.7729, -0.0411, -0.0171, -0.0087, 0.0242 };
          bnCC = new double[] { 0.0000, -0.0324, 0.0147, -0.0103, 0.0210 };
          break;
        case Location.Naha:
          anDBT = new double[] { 0.0091, -5.3214, -0.2766, -0.1536, 0.1923 };
          bnDBT = new double[] { 0.0000, 2.9532, 0.0588, 0.1686, -0.0156 };
          anHRT = new double[] { 0.0071, -5.6110, 0.0933, -0.0870, 0.1813 };
          bnHRT = new double[] { 0.0000, 2.1287, -0.3409, 0.0453, 0.1980 };
          anRAD = new double[] { 0.0019, 0.0017, 0.0121, -0.0082, 0.0056 };
          bnRAD = new double[] { 0.0000, 0.0724, 0.0061, 0.0043, -0.0122 };
          anFF = new double[] { 0.7991, -0.0288, -0.0036, 0.0057, 0.0024 };
          bnFF = new double[] { 0.0000, 0.0727, -0.0097, 0.0093, -0.0065 };
          anCC = new double[] { 0.7882, 0.0221, -0.0010, 0.0026, -0.0179 };
          bnCC = new double[] { 0.0000, -0.0490, 0.0134, -0.0110, 0.0071 };
          break;
        default://東京
          anDBT = new double[] { 0.0191, -9.1951, -0.5818, 0.6386, -0.4619 };
          bnDBT = new double[] { 0.0000, 4.5427, -0.3686, 0.5093, -0.2657 };
          anHRT = new double[] { 0.0139, -5.8197, 0.2694, 0.3759, -0.1934 };
          bnHRT = new double[] { 0.0000, 2.9745, -1.2801, 0.4618, -0.0777 };
          anRAD = new double[] { 0.0165, 0.1610, 0.0332, 0.0231, -0.0285 };
          bnRAD = new double[] { 0.0000, -0.0061, 0.0052, 0.0085, -0.0225 };
          anFF = new double[] { 0.8132, 0.0770, 0.0077, 0.0152, -0.0207 };
          bnFF = new double[] { 0.0000, 0.0002, -0.0101, 0.0192, -0.0217 };
          anCC = new double[] { 0.7975, -0.0427, -0.0228, -0.0255, 0.0199 };
          bnCC = new double[] { 0.0000, -0.0055, 0.0156, -0.0112, 0.0037 };
          break;
      }

      //日別年周期データを作成
      dbt = new double[365];
      hrt = new double[365];
      rad = new double[365];
      cc = new double[365];
      double[] ff = new double[365];
      for (int i = 0; i < anDBT.Length; i++)
      {
        double wk = 2d * Math.PI * i / 365d;
        for (int j = 0; j < 365; j++)
        {
          double cwk = Math.Cos(wk * j);
          double swk = Math.Sin(wk * j);
          dbt[j] += anDBT[i] * cwk - bnDBT[i] * swk;
          hrt[j] += anHRT[i] * cwk - bnHRT[i] * swk;
          rad[j] += anRAD[i] * cwk - bnRAD[i] * swk;
          ff[j] += anFF[i] * cwk - bnFF[i] * swk;
          cc[j] += anCC[i] * cwk - bnCC[i] * swk;
        }
      }
      for (int i = 0; i < 365; i++)
        cc[i] = (1 - ff[i]) / (2 - ff[i] - cc[i]);
    }

    private static void makeHourlyDataWithFFT
      (out double[] dbt, out double[] hrt, out double[] rad, out double[] cc10, out double[] cc00)
    {
      double[] anDBT, bnDBT, anHRT, bnHRT, anRAD, bnRAD, anCC10, bnCC10, anCC00, bnCC00;
      switch (LOCATION)
      {
        case Location.Osaka:
          anDBT = new double[] { 0.0000, -1.9006, 0.6382, -0.0646 };
          bnDBT = new double[] { 0.0000, 1.9080, -0.3301, -0.0937 };
          anHRT = new double[] { 0.0000, 0.0358, 0.0370, 0.0001 };
          bnHRT = new double[] { 0.0000, -0.0235, 0.0655, -0.0105 };
          anRAD = new double[] { 0.2175, 0.3078, 0.0175, -0.0038 };
          bnRAD = new double[] { 0.0000, -0.0037, 0.0007, -0.0052 };
          anCC10 = new double[] { 0.1261, 0.4997, 0.0427, -0.0507 };
          bnCC10 = new double[] { 0.0000, -0.0082, -0.0007, -0.0005 };
          anCC00 = new double[] { 0.3963, 0.2935, 0.0120, 0.0129 };
          bnCC00 = new double[] { 0.0000, -0.0046, 0.0008, -0.0047 };
          break;
        case Location.Sapporo:
          anDBT = new double[] { 0.0000, -2.0102, 0.6600, -0.0203 };
          bnDBT = new double[] { 0.0000, 1.2000, -0.0492, -0.1270 };
          anHRT = new double[] { 0.0000, -0.1166, 0.0264, 0.0067 };
          bnHRT = new double[] { 0.0000, 0.1300, 0.0315, -0.0058 };
          anRAD = new double[] { 0.1537, 0.2220, 0.0145, 0.0137 };
          bnRAD = new double[] { 0.0000, 0.0231, 0.0127, -0.0153 };
          anCC10 = new double[] { 0.0772, 0.4520, 0.0662, -0.0249 };
          bnCC10 = new double[] { 0.0000, 0.0431, 0.0163, -0.0144 };
          anCC00 = new double[] { 0.3787, 0.2029, 0.0115, 0.0138 };
          bnCC00 = new double[] { 0.0000, 0.0036, 0.0034, -0.0043 };
          break;
        case Location.Sendai:
          anDBT = new double[] { 0.0000, -1.9449, 0.7034, -0.0493 };
          bnDBT = new double[] { 0.0000, 1.3603, -0.1132, -0.1271 };
          anHRT = new double[] { 0.0000, -0.0205, 0.0410, -0.0030 };
          bnHRT = new double[] { 0.0000, 0.1717, 0.0334, -0.0134 };
          anRAD = new double[] { 0.1554, 0.2151, 0.0000, -0.0013 };
          bnRAD = new double[] { 0.0000, 0.0141, 0.0000, -0.0068 };
          anCC10 = new double[] { 0.0180, 0.3333, 0.0000, -0.0400 };
          bnCC10 = new double[] { 0.0000, 0.0282, 0.0000, -0.0099 };
          anCC00 = new double[] { 0.3852, 0.2452, 0.0000, 0.0117 };
          bnCC00 = new double[] { 0.0000, 0.0005, 0.0000, 0.0002 };
          break;
        case Location.Fukuoka:
          anDBT = new double[] { 0.0000, -1.7793, 0.6353, -0.0995 };
          bnDBT = new double[] { 0.0000, 1.6188, -0.2374, -0.1190 };
          anHRT = new double[] { 0.0000, -0.1133, 0.0500, 0.0032 };
          bnHRT = new double[] { 0.0000, 0.2190, -0.0122, 0.0042 };
          anRAD = new double[] { 0.1863, 0.2561, 0.0000, -0.0091 };
          bnRAD = new double[] { 0.0000, -0.0181, 0.0000, 0.0034 };
          anCC10 = new double[] { 0.0040, 0.3535, 0.0000, -0.0401 };
          bnCC10 = new double[] { 0.0000, -0.0277, 0.0000, 0.0049 };
          anCC00 = new double[] { 0.3579, 0.1904, 0.0000, 0.0053 };
          bnCC00 = new double[] { 0.0000, -0.0113, 0.0000, 0.0056 };
          break;
        case Location.Naha:
          anDBT = new double[] { 0.0000, -1.1943, 0.4402, -0.0188 };
          bnDBT = new double[] { 0.0000, 0.7588, -0.1384, -0.0821 };
          anHRT = new double[] { 0.0000, -0.1003, 0.0246, 0.0117 };
          bnHRT = new double[] { 0.0000, -0.0056, 0.0006, 0.0013 };
          anRAD = new double[] { 0.2241, 0.3052, 0.0000, -0.0039 };
          bnRAD = new double[] { 0.0000, -0.0452, 0.0000, 0.0008 };
          anCC10 = new double[] { 0.0626, 0.3517, 0.0000, -0.0314 };
          bnCC10 = new double[] { 0.0000, -0.0491, 0.0000, 0.0113 };
          anCC00 = new double[] { 0.3428, 0.2665, 0.0000, 0.0165 };
          bnCC00 = new double[] { 0.0000, -0.0426, 0.0000, -0.0061 };
          break;
        default:
          anDBT = new double[] { 0.0000, -1.5906, 0.5391, -0.0523 };
          bnDBT = new double[] { 0.0000, 1.7772, -0.2794, -0.0602 };
          anHRT = new double[] { 0.0000, 0.1410, -0.0029, -0.0107 };
          bnHRT = new double[] { 0.0000, 0.1681, 0.0217, -0.0091 };
          anRAD = new double[] { 0.1787, 0.2445, 0.0000, -0.0139 };
          bnRAD = new double[] { 0.0000, 0.0184, 0.0000, -0.0036 };
          anCC10 = new double[] { 0.0450, 0.3672, 0.0000, -0.0390 };
          bnCC10 = new double[] { 0.0000, 0.0230, 0.0000, -0.0037 };
          anCC00 = new double[] { 0.3807, 0.2269, 0.0000, -0.0008 };
          bnCC00 = new double[] { 0.0000, 0.0062, 0.0000, -0.0019 };
          break;
      }

      //時刻別データを作成
      dbt = new double[24];
      hrt = new double[24];
      rad = new double[24];
      cc10 = new double[24];
      cc00 = new double[24];
      for (int i = 0; i < anDBT.Length; i++)
      {
        double wk = 2d * Math.PI * i / 24d;
        for (int j = 0; j < 24; j++)
        {
          double cwk = Math.Cos(wk * j);
          double swk = Math.Sin(wk * j);
          dbt[j] += anDBT[i] * cwk - bnDBT[i] * swk;
          hrt[j] += anHRT[i] * cwk - bnHRT[i] * swk;
          rad[j] += anRAD[i] * cwk - bnRAD[i] * swk;
          cc10[j] += anCC10[i] * cwk - bnCC10[i] * swk;
          cc00[j] += anCC00[i] * cwk - bnCC00[i] * swk;
        }
      }
    }

    private static void interpolate365Data
      (double[] dbt365, double[] hrt365, double[] rad365, double[] cc365,
      out double[] dbt, out double[] hrt, out double[] rad, out double[] cc)
    {
      int day = dbt365.Length;

      double[] dbt367 = new double[day + 2];
      double[] hrt367 = new double[day + 2];
      double[] rad367 = new double[day + 2];
      double[] cc367 = new double[day + 2];
      dbt365.CopyTo(dbt367, 1);
      hrt365.CopyTo(hrt367, 1);
      rad365.CopyTo(rad367, 1);
      cc365.CopyTo(cc367, 1);
      dbt367[day + 1] = dbt367[1];
      hrt367[day + 1] = hrt367[1];
      rad367[day + 1] = rad367[1];
      cc367[day + 1] = cc367[1];
      dbt367[0] = dbt367[day];
      hrt367[0] = hrt367[day];
      rad367[0] = rad367[day];
      cc367[0] = cc367[day];

      //スプライン補間
      double[] sx = new double[day + 2];
      double[] hx = new double[day * 24];
      for (int i = 0; i < sx.Length; i++) sx[i] = -12 + 24 * i;
      for (int i = 0; i < hx.Length; i++) hx[i] = i;
      double[] cDBT = CubicSpline.GetParameters(sx, dbt367);
      double[] cHRT = CubicSpline.GetParameters(sx, hrt367);
      double[] cRAD = CubicSpline.GetParameters(sx, rad367);
      double[] cCC = CubicSpline.GetParameters(sx, cc367);
      dbt = CubicSpline.Interpolate(sx, dbt367, cDBT, hx);
      hrt = CubicSpline.Interpolate(sx, hrt367, cHRT, hx);
      rad = CubicSpline.Interpolate(sx, rad367, cRAD, hx);
      cc = CubicSpline.Interpolate(sx, cc367, cCC, hx);
    }

    #endregion    

    #region 振幅変調係数の計算

    private static void makeSwingCoefficient()
    {
      //標準振幅
      double swingDBT, swingHRT;
      switch (LOCATION)
      {
        case Location.Osaka:
          swingDBT = 5.262;
          swingHRT = 0.530;
          break;
        case Location.Sapporo:
          swingDBT = 5.084;
          swingHRT = 0.364;
          break;
        case Location.Sendai:
          swingDBT = 5.266;
          swingHRT = 0.384;
          break;
        case Location.Fukuoka:
          swingDBT = 5.262;
          swingHRT = 0.530;
          break;
        case Location.Naha:
          swingDBT = 3.155;
          swingHRT = 0.195;
          break;
        default://東京
          swingDBT = 5.179;
          swingHRT = 0.454;
          break;
      }


      //確定成分の計算
      double[] hDBT365, hHRT365, hRAD365, hCC365;
      double[] syDBT, syHRT, syRAD, syCC, sdDBT, sdHRT, sdRAD, sdCC10, sdCC00;
      makeDailyDataWithFFT(out hDBT365, out hHRT365, out hRAD365, out hCC365);
      interpolate365Data(hDBT365, hHRT365, hRAD365, hCC365, out syDBT, out syHRT, out syRAD, out syCC);
      makeHourlyDataWithFFT(out sdDBT, out sdHRT, out sdRAD, out sdCC10, out sdCC00);

      using (StreamWriter sWriter = new StreamWriter("swingCoef.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamReader sReader = new StreamReader("outH_Rad_Trend.csv", Encoding.GetEncoding("Shift_JIS")))
      {
        sReader.ReadLine();
        sWriter.WriteLine("日付,平均大気透過率,乾球温度シフト,日較差比,絶対湿度シフト,日較絶対湿度差比,");

        string buff1;
        DateTime dt = new DateTime(1982, 1, 1, 0, 0, 0);
        int hour = 0;
        double sumDBT = 0;
        double minDBT = 999;
        double maxDBT = -999;
        double sumHRT = 0;
        double minHRT = 999;
        double maxHRT = -999;
        double sRAD = 0;
        int nRAD = 0;
        while ((buff1 = sReader.ReadLine()) != null)
        {
          string[] buff2 = buff1.Split(',');

          double dbt = double.Parse(buff2[8]) - syDBT[hour];
          double hrt = double.Parse(buff2[9]) - syHRT[hour];
          sumDBT += (dbt - sdDBT[hour % 24]);
          sumHRT += (hrt - sdHRT[hour % 24]);
          double rad = double.Parse(buff2[10]);
          if (rad != 0)
          {
            double rr = syCC[hour] * sdCC10[dt.Hour] + (1 - syCC[hour]) * sdCC00[dt.Hour];
            if (int.Parse(buff2[11]) == 10) sRAD += (rad - (syRAD[hour] + rr));
            else sRAD += (rad - (syRAD[hour] + rr));
            
            nRAD++;
          }
          maxDBT = Math.Max(dbt, maxDBT);
          minDBT = Math.Min(dbt, minDBT);
          maxHRT = Math.Max(hrt, maxHRT);
          minHRT = Math.Min(hrt, minHRT);

          if (dt.Hour == 23)
          {
            sWriter.Write(dt.ToShortDateString() + ",");
            sWriter.Write((sRAD / nRAD) + ",");
            sWriter.Write((sumDBT / 24d) + ",");
            sWriter.Write(((maxDBT - minDBT) / swingDBT) + ",");
            sWriter.Write((sumHRT / 24d) + ",");
            sWriter.Write(((maxHRT - minHRT) / swingHRT));
            sWriter.WriteLine();

            sumDBT = 0;
            sumHRT = 0;
            minDBT = 999;
            maxDBT = -999;
            minHRT = 999;
            maxHRT = -999;
            sRAD = nRAD = 0;
          }

          hour++;
          if (hour == 8760) hour = 0;

          dt = dt.AddHours(1);
          //うるう年対応
          if (dt.Month == 2 && dt.Day == 29) dt = dt.AddDays(1);
        }
      }
    }

    #endregion

    #region ランダム項の分離

    private static void removeRandom()
    {
      double cDBT_A1, cDBT_A2, cDBT_B2;
      double cHRT_A1, cHRT_A2, cHRT_B2;
      if (true)
      {
        switch (LOCATION)
        {
          case Location.Osaka:
            cDBT_A1 = 0;
            cDBT_A2 = 1.486;
            cDBT_B2 = 1.337;
            cHRT_A1 = -4.173;
            cHRT_A2 = 0;
            cHRT_B2 = 1;
            break;
          case Location.Sapporo:
            cDBT_A1 = 0.723;
            cDBT_A2 = 1.346;
            cDBT_B2 = 1.343;
            cHRT_A1 = -2.034;
            cHRT_A2 = 0;
            cHRT_B2 = 1;
            break;
          case Location.Sendai:
            cDBT_A1 = 1.794;
            cDBT_A2 = 1.284;
            cDBT_B2 = 1.304;
            cHRT_A1 = -2.182;
            cHRT_A2 = 0;
            cHRT_B2 = 1;
            break;
          case Location.Fukuoka:
            cDBT_A1 = -0.162;
            cDBT_A2 = 1.324;
            cDBT_B2 = 1.290;
            cHRT_A1 = -3.323;
            cHRT_A2 = 0;
            cHRT_B2 = 1;
            break;
          case Location.Naha:
            cDBT_A1 = 1.899;
            cDBT_A2 = 1.120;
            cDBT_B2 = 1.332;
            cHRT_A1 = -2.323;
            cHRT_A2 = 0;
            cHRT_B2 = 1;
            break;
          default://東京
            cDBT_A1 = 2.584;
            cDBT_A2 = 1.118;
            cDBT_B2 = 1.249;
            cHRT_A1 = -2.806;
            cHRT_A2 = 0;
            cHRT_B2 = 1;
            break;
        }
      }
      else
      {
        cDBT_A1 = 0;
        cDBT_A2 = 0;
        cDBT_B2 = 1;
        cHRT_A1 = 0;
        cHRT_A2 = 0;
        cHRT_B2 = 1;
      }

      //確定成分の計算
      double[] syDBT365, syHRT365, syRAD365, syCC365, sdDBT, sdHRT, sdRAD, sdCC10, sdCC00;
      makeDailyDataWithFFT(out syDBT365, out syHRT365, out syRAD365, out syCC365);
      makeHourlyDataWithFFT(out sdDBT, out sdHRT, out sdRAD, out sdCC10, out sdCC00);
      double[] syDBT, syHRT, syRAD, syCC;
      interpolate365Data(syDBT365, syHRT365, syRAD365, syCC365, out syDBT, out syHRT, out syRAD, out syCC);

      //大気透過率確率成分の計算
      List<double> iRAD_ls = new List<double>();
      using (StreamReader sReader = new StreamReader("outH_Rad_Trend.csv", Encoding.GetEncoding("Shift_JIS")))
      {
        sReader.ReadLine();
        DateTime dt = new DateTime(1982, 1, 1, 0, 0, 0);
        string buff1;
        double sumRAD = 0;
        int nRAD = 0;
        int hour = 0;
        while ((buff1 = sReader.ReadLine()) != null)
        {
          string[] buff2 = buff1.Split(',');

          double rad = double.Parse(buff2[10]);
          if (rad != 0)
          {
            double rr = syCC[hour] * sdCC10[dt.Hour] + (1 - syCC[hour]) * sdCC00[dt.Hour];
            sumRAD += (rad - (syRAD365[hour / 24] + rr));
            nRAD++;
          }
          if (dt.Hour == 23)
          {
            iRAD_ls.Add(sumRAD / nRAD);
            sumRAD = nRAD = 0;
          }
          hour++;
          if (hour == 8760) hour = 0;

          dt = dt.AddHours(1);
          //うるう年対応
          if (dt.Month == 2 && dt.Day == 29) dt = dt.AddDays(1);
        }
      }
      double[] iRAD = iRAD_ls.ToArray();

      //乾球温度と絶対湿度の確定成分作成
      double[] syDBT365_2 = new double[iRAD.Length];
      double[] syHRT365_2 = new double[iRAD.Length];
      double[] syRAD365_2 = new double[iRAD.Length];
      double[] syCC365_2 = new double[iRAD.Length];
      for (int i = 0; i < syDBT365_2.Length; i++)
      {
        syDBT365_2[i] = syDBT365[i % 365] + (iRAD[i] * cDBT_A1);
        syHRT365_2[i] = syHRT365[i % 365] + (iRAD[i] * cHRT_A1);
        syRAD365_2[i] = syRAD365[i % 365];
        syCC365_2[i] = syCC365[i % 365];
      }
      interpolate365Data(syDBT365_2, syHRT365_2, syRAD365_2, syCC365_2, out syDBT, out syHRT, out syRAD, out syCC);

      //日較差補正
      for (int i = 0; i < syDBT.Length; i++)
      {
        int day = i / 24;
        int hour = i % 24;
        syDBT[i] += sdDBT[hour] * (cDBT_A2 * iRAD[day] + cDBT_B2);
        syHRT[i] += sdHRT[hour] * (cHRT_A2 * iRAD[day] + cHRT_B2);
      }

      using (StreamWriter sWriter = new StreamWriter("random.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamReader sReader = new StreamReader("outH_Rad_Trend.csv", Encoding.GetEncoding("Shift_JIS")))
      {
        sReader.ReadLine();
        sWriter.WriteLine
          ("日付,時刻,乾球温度,絶対湿度,大気透過率,乾球温度S(t),絶対湿度S(t),大気透過率S(t),乾球温度I(t),絶対湿度I(t),大気透過率I(t)1,大気透過率I(t)2");

        string buff1;

        int hour = 0;
        double siDBT, siHRT, siRAD;
        DateTime dt = new DateTime(1982, 1, 1, 0, 0, 0);
        while ((buff1 = sReader.ReadLine()) != null)
        {
          string[] buff2 = buff1.Split(',');
          siDBT = double.Parse(buff2[8]);
          siHRT = double.Parse(buff2[9]);
          siRAD = double.Parse(buff2[10]);

          double sRAD1 = 0;
          double sRAD2 = 0;
          if (siRAD != 0)
          {
            if (int.Parse(buff2[11]) == 10) sRAD1 = syRAD[hour] + sdCC10[hour % 24];
            else sRAD1 = syRAD[hour] + sdCC00[hour % 24];
            double rr = syCC[hour] * sdCC10[dt.Hour] + (1 - syCC[hour]) * sdCC00[dt.Hour];
            sRAD2 = syRAD[hour] + rr;
          }

          sWriter.Write(dt.ToShortDateString() + "," + dt.Hour + ",");
          sWriter.Write(siDBT + "," + siHRT + "," + siRAD + ",");
          sWriter.Write(syDBT[hour] + "," + syHRT[hour] + "," + sRAD1 + ",");
          sWriter.WriteLine((siDBT - syDBT[hour]) + "," + (siHRT - syHRT[hour]) + "," + (siRAD - sRAD1) + "," + (siRAD - sRAD2));

          hour++;
          dt = dt.AddHours(1);
          //うるう年対応
          if (dt.Month == 2 && dt.Day == 29) dt = dt.AddDays(1);
        }
      }

      using (StreamWriter sWriter = new StreamWriter("random_daily.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamReader sReader = new StreamReader("random.csv", Encoding.GetEncoding("Shift_JIS")))
      {
        sReader.ReadLine();
        sWriter.WriteLine("日付,乾球温度,絶対湿度,大気透過率,乾球温度I(t),絶対湿度I(t),大気透過率I(t)1,大気透過率I(t)2,乾球温度I(t)標準偏差,絶対湿度I(t)標準偏差");

        string buff1;
        double aveDBTsi, aveHRTsi, aveRADsi, aveDBTi, aveHRTi, aveRADi1, aveRADi2;
        double[] sdvDBT = new double[24];
        double[] sdvHRT = new double[24];
        int nRAD = 0;
        aveDBTsi = aveHRTsi = aveRADsi = aveDBTi = aveHRTi = aveRADi1 = aveRADi2 = 0;
        int hour = 0;
        while ((buff1 = sReader.ReadLine()) != null)
        {
          string[] buff2 = buff1.Split(',');

          aveDBTsi += double.Parse(buff2[2]);
          aveHRTsi += double.Parse(buff2[3]);
          aveDBTi += double.Parse(buff2[8]);
          sdvDBT[hour] = double.Parse(buff2[8]);
          aveHRTi += double.Parse(buff2[9]);
          sdHRT[hour] = double.Parse(buff2[9]);

          if (double.Parse(buff2[4]) != 0)
          {
            aveRADsi += double.Parse(buff2[4]);
            aveRADi1 += double.Parse(buff2[10]);
            aveRADi2 += double.Parse(buff2[11]);
            nRAD++;
          }
          hour++;
          if (hour == 24)
          {
            //標準偏差を計算
            aveDBTi /= 24d;
            aveHRTi /= 24d;
            double sdvDBTi = 0;
            double sdvHRTi = 0;
            for (int i = 0; i < 24; i++)
            {
              sdvDBTi += (sdvDBT[i] - aveDBTi) * (sdvDBT[i] - aveDBTi);
              sdvHRTi += (sdvHRT[i] - aveHRTi) * (sdvHRT[i] - aveHRTi);
            }
            sdvDBTi = Math.Sqrt(sdvDBTi / 24d);
            sdvHRTi = Math.Sqrt(sdvHRTi / 24d);

            sWriter.Write(buff2[0] + ",");
            sWriter.Write((aveDBTsi / 24d) + ",");
            sWriter.Write((aveHRTsi / 24d) + ",");
            sWriter.Write((aveRADsi / nRAD) + ",");
            sWriter.Write(aveDBTi + ",");
            sWriter.Write(aveHRTi + ",");
            sWriter.Write((aveRADi1 / nRAD) + ",");
            sWriter.Write((aveRADi2 / nRAD) + ",");
            sWriter.Write(sdvDBTi + ",");
            sWriter.WriteLine(sdvHRTi + ",");

            aveDBTsi = aveHRTsi = aveRADsi = aveDBTi = aveHRTi = aveRADi1 = aveRADi2 = nRAD = 0;
            hour = 0;
          }
        }
      }

      using (StreamWriter sWriter = new StreamWriter("random_STD.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamReader sReader = new StreamReader("random_daily.csv", Encoding.GetEncoding("Shift_JIS")))
      {
        sReader.ReadLine();

        //1991年まで飛ばす
        for (int i = 0; i < 365 * 9; i++) sReader.ReadLine();

        string buff1;
        int day = 0;
        double[] dbtSTD = new double[365];
        double[] hrtSTD = new double[365];
        int year = 0;
        while ((buff1 = sReader.ReadLine()) != null)
        {
          string[] buff2 = buff1.Split(',');
          dbtSTD[day] += double.Parse(buff2[8]);
          hrtSTD[day] += double.Parse(buff2[9]);

          day++;
          if (day == 365)
          {
            day = 0;
            year++;
          }
        }
        sWriter.WriteLine("日にち,乾球温度標準偏差,絶対湿度標準偏差");
        for (int i = 0; i < 365; i++)
        {
          sWriter.WriteLine((i + 1) + "," + (dbtSTD[i] / year) + "," + (hrtSTD[i] / year));
        }
      }

    }

    #endregion

    #region 大気透過率ARモデル作成

    private static void makeARData_CC()
    {
      const int MAX_ORDER = 7;

      StreamWriter[] swAR = new StreamWriter[MAX_ORDER];
      for (int i = 0; i < swAR.Length; i++)
        swAR[i] = new StreamWriter("ARData\\ARData_CC" + (i+1) + ".csv", false, Encoding.GetEncoding("Shift_JIS"));

      using (StreamWriter sWriter1 = new StreamWriter("ARData_CC.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamReader sReader1 = new StreamReader("Unryo00And10.csv", Encoding.GetEncoding("Shift_JIS")))
      {
        string sb1;
        while ((sb1 = sReader1.ReadLine()) != null)
        {
          string[] sb2 = sb1.Split(',');
          int start = 0;
          int end = 0;
          for (int i = 1; i < sb2.Length; i++)
          {
            if (sb2[i] != " ")
            {
              start = i;
              break;
            }
          }
          for (int i = sb2.Length - 2; 0 < i; i--)
          {
            if (sb2[i] != " ")
            {
              end = i;
              break;
            }
          }

          //最大次数
          int maxOrder = Math.Min(end - start, MAX_ORDER);
          for (int i = 1; i < maxOrder + 1; i++)
          {
            StreamWriter sw = swAR[i - 1];
            int dNum = maxOrder - i + 1;
            for (int j = 0; j < dNum; j++)
            {
              for (int k = 0; k <= i; k++)
              {
                if(k == 0) sw.Write(sb2[end - j - k]);
                else sw.Write("," + sb2[end - j - k]);
              }
              sw.WriteLine();
            }
          }
        }
      }
      for (int i = 0; i < swAR.Length; i++)swAR[i].Close();
    }

    private static void makeARModel_CC()
    {
      const int MAX_ORDER = 7;

      StreamReader[] srAR = new StreamReader[MAX_ORDER];
      for (int i = 0; i < srAR.Length; i++)
        srAR[i] = new StreamReader("ARData\\ARData_CC" + (i + 1) + ".csv", Encoding.GetEncoding("Shift_JIS"));

      using (StreamWriter sWriter = new StreamWriter("ARData\\Param_CC.csv", false, Encoding.GetEncoding("Shift_JIS")))
      {
        for (int i = 6; i < MAX_ORDER; i++)
        {
          //データ読み込み
          List<double> y = new List<double>();
          List<double[]> x = new List<double[]>();
          string sb1;
          while ((sb1 = srAR[i - 1].ReadLine()) != null)
          {
            string[] sb2 = sb1.Split(',');
            y.Add(double.Parse(sb2[0]));
            double[] xx = new double[i];
            for (int j = 0; j < xx.Length; j++) xx[j] = double.Parse(sb2[j]);
            x.Add(xx);
          }

          //最小二乗法
          double[] yV = y.ToArray();
          double[,] xM = new double[yV.Length, i];
          for (int j = 0; j < xM.GetLength(0); j++)
          {
            for (int k = 0; k < i; k++)
            {
              xM[j, k] = x[j][k];
            }
          }

          double sig2, aig;
          double[] a = LinearAlgebra.LeastSquareFit(yV, xM, out sig2, out aig);
        }
      }

      for (int i = 0; i < srAR.Length; i++) srAR[i].Close();
    }

    #endregion

    #region VARモデル入力データの作成

    private static void makeVARInputs()
    {
      double[] anDBT, bnDBT, anHRT, bnHRT;
      switch (LOCATION)
      {
        case Location.Osaka:
          anDBT = new double[] { 1.3801, 0.1494, 0.0147 };
          bnDBT = new double[] { 0.0000, -0.1642, 0.0041 };
          anHRT = new double[] { 1.2499, -0.3970, -0.1582 };
          bnHRT = new double[] { 0.0000, 0.1771, 0.1389 };
          break;
        case Location.Sapporo:
          anDBT = new double[] { 1.5794, 0.4146, 0.0894 };
          bnDBT = new double[] { 0.0000, -0.0883, 0.1445 };
          anHRT = new double[] { 0.9420, -0.3845, -0.0669 };
          bnHRT = new double[] { 0.0000, 0.3613, -0.0884 };
          break;
        case Location.Sendai:
          anDBT = new double[] { 1.4371, 0.0498, 0.0356 };
          bnDBT = new double[] { 0.0000, -0.1038, 0.1049 };
          anHRT = new double[] { 1.1640, -0.5124, -0.1287 };
          bnHRT = new double[] { 0.0000, 0.3059, -0.0090 };
          break;
        case Location.Fukuoka:
          anDBT = new double[] { 1.3338, 0.1684, 0.1002 };
          bnDBT = new double[] { 0.0000, -0.2047, 0.0312 };
          anHRT = new double[] { 1.2744, -0.1366, -0.1652 };
          bnHRT = new double[] { 0.0000, 0.1132, 0.1945 };
          break;
        case Location.Naha:
          anDBT = new double[] { 0.8363, 0.2361, 0.0041 };
          bnDBT = new double[] { 0.0000, -0.1779, -0.0364 };
          anHRT = new double[] { 1.9009, 0.3594, -0.4965 };
          bnHRT = new double[] { 0.0000, -0.3534, 0.4481 };
          break;
        default:
          anDBT = new double[] { 1.2483, 0.0497, -0.0513 };
          bnDBT = new double[] { 0.0000, -0.1111, 0.0172 };
          anHRT = new double[] { 1.3130, -0.4424, -0.1839 };
          bnHRT = new double[] { 0.0000, 0.2039, 0.0424 };
          break;
      }

      //日別年周期データを作成
      double[] dbtSIG = new double[365];
      double[] hrtSIG = new double[365];
      for (int i = 0; i < anDBT.Length; i++)
      {
        double wk = 2d * Math.PI * i / 365d;
        for (int j = 0; j < 365; j++)
        {
          double cwk = Math.Cos(wk * j);
          double swk = Math.Sin(wk * j);
          dbtSIG[j] += anDBT[i] * cwk - bnDBT[i] * swk;
          hrtSIG[j] += anHRT[i] * cwk - bnHRT[i] * swk;
        }
      }

      //時刻別データに変換
      dbtSIG = interpolateDailyData(dbtSIG);
      hrtSIG = interpolateDailyData(hrtSIG);

      using (StreamWriter sWriter = new StreamWriter("random2.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamReader sReader = new StreamReader("random.csv", Encoding.GetEncoding("Shift_JIS")))
      {
        sWriter.WriteLine("日付,時刻,乾球温度,絶対湿度,乾球温度標準偏差調整,絶対湿度標準偏差調整");
        sReader.ReadLine();
        string buff1;
        int hour = 0;
        while ((buff1 = sReader.ReadLine()) != null)
        {
          string[] buff2 = buff1.Split(',');
          double dbtRND2 = double.Parse(buff2[8]) / dbtSIG[hour];
          double hrtRND2 = double.Parse(buff2[9]) / hrtSIG[hour];

          sWriter.WriteLine(buff2[0] + "," + buff2[1] + "," + buff2[8] + "," + buff2[9] + "," + dbtRND2 + "," + hrtRND2);

          hour++;
          if (hour == 8760) hour = 0;
        }
      }
    }

    /// <summary>日平均データをSpline補間して時間平均データを作成する</summary>
    /// <param name="dailyData">日平均データ</param>
    /// <returns>時間平均データ</returns>
    private static double[] interpolateDailyData(double[] dailyData)
    {
      double[] dd = new double[dailyData.Length + 2];
      dailyData.CopyTo(dd, 1);
      dd[0] = dd[dd.Length - 2];
      dd[dd.Length - 1] = dd[1];
      double[] sx = new double[dailyData.Length + 2];
      double[] hx = new double[dailyData.Length * 24];
      for (int i = 0; i < sx.Length; i++) sx[i] = -12 + 24 * i;
      for (int i = 0; i < hx.Length; i++) hx[i] = i;
      double[] cDD = CubicSpline.GetParameters(sx, dd);
      return CubicSpline.Interpolate(sx, dd, cDD, hx);
    }

    #endregion

    #region ARモデル残差変動の計算

    private static void makeARResidual()
    {
      double[] iDBTCof, iHRTCof;
      switch (LOCATION)
      {
        case Location.Osaka:
          iDBTCof = new double[] { 1.1785, 0.0852, -0.0937, -0.0016, -0.1435, -0.0318 };
          iHRTCof = new double[] { -0.0122, 1.0342, 0.0403, -0.0518, -0.0119, -0.0279 };
          break;
        case Location.Naha:
          iDBTCof = new double[] { 0.9684, 0.0464, -0.0321, 0.0105, -0.0141, -0.0143 };
          iHRTCof = new double[] { 0.0102, 0.8534, 0.0331, 0.0997, -0.0054, 0.0086 };
          break;
        default:
          iDBTCof = new double[] { 1.1385, 0.0719, -0.0936, 0.0080, -0.0980, -0.0268 };
          iHRTCof = new double[] { 0.0176, 1.0087, 0.0233, -0.0456, -0.0274, -0.0089 };
          break;
      }
    
      using (StreamWriter sWriter1 = new StreamWriter("randomRemainDBT.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamWriter sWriter2 = new StreamWriter("randomRemainHRT.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamReader sReader = new StreamReader("random.csv", Encoding.GetEncoding("Shift_JIS")))
      {
        sReader.ReadLine();
        //1982-1990年まで飛ばす
        for (int i = 0; i < 8760 * 9 - 3; i++)
          sReader.ReadLine();

        //データ読み込み
        double[] rndDBT = new double[8760 * 9 + 3];
        double[] rndHRT = new double[8760 * 9 + 3];
        string sb;
        int index = 0;
        while ((sb = sReader.ReadLine()) != null)
        {
          string[] sb2 = sb.Split(',');
          rndDBT[index] = double.Parse(sb2[8]);
          rndHRT[index] = double.Parse(sb2[9]);
          index++;
        }

        //ARモデル残差計算
        for (int i = rndDBT.Length - 1; 2 < i; i--)
        {
          rndDBT[i] -=
            rndDBT[i - 1] * iDBTCof[0] + rndHRT[i - 1] * iDBTCof[1] +
            rndDBT[i - 2] * iDBTCof[2] + rndHRT[i - 2] * iDBTCof[3] +
            rndDBT[i - 3] * iDBTCof[4] + rndHRT[i - 3] * iDBTCof[5];
          rndHRT[i] -=
            rndDBT[i - 1] * iHRTCof[0] + rndHRT[i - 1] * iHRTCof[1] +
            rndDBT[i - 2] * iHRTCof[2] + rndHRT[i - 2] * iHRTCof[3] +
            rndDBT[i - 3] * iHRTCof[4] + rndHRT[i - 3] * iHRTCof[5];
        }

        for (int i = 0; i < 365; i++)
        {
          sWriter1.Write(i + ",");
          sWriter2.Write(i + ",");
          for (int j = 0; j < 9; j++)
          {
            for (int k = 0; k < 24; k++)
            {
              sWriter1.Write(rndDBT[j * 8760 + i * 24 + k + 3] + ",");
              sWriter2.Write(rndHRT[j * 8760 + i * 24 + k + 3] + ",");
            }
          }
          sWriter1.WriteLine();
          sWriter2.WriteLine();
        }
      }
    }
  

    #endregion

    #region テスト処理

    private static void test()
    {
      DateTime dt = new DateTime(1981, 1, 1, 0, 0, 0);
      Sun sun;
      double[] dbt, hrt, rad;
      bool[] isFair;
      switch (LOCATION)
      {
        case Location.Osaka:
          sun = new Sun(34, 40.7, 0, 135, 31.3, 0, 135, 0, 0);
          RandomWeather.MakeWeather(DateTime.Now.Millisecond, RandomWeather.Location.Osaka, 20, out dbt, out hrt, out rad, out isFair);
          break;
        case Location.Sapporo:
          sun = new Sun(43, 3.5, 0, 141, 19.9, 0, 135, 0, 0);
          RandomWeather.MakeWeather(DateTime.Now.Millisecond, RandomWeather.Location.Sapporo, 20, out dbt, out hrt, out rad, out isFair);
          break;
        case Location.Sendai:
          sun = new Sun(38, 15.5, 0, 140, 54.0, 0, 135, 0, 0);
          RandomWeather.MakeWeather(DateTime.Now.Millisecond, RandomWeather.Location.Sendai, 20, out dbt, out hrt, out rad, out isFair);
          break;
        case Location.Fukuoka:
          sun = new Sun(33, 34.8, 0, 130, 22.6, 0, 135, 0, 0);
          RandomWeather.MakeWeather(DateTime.Now.Millisecond, RandomWeather.Location.Fukuoka, 20, out dbt, out hrt, out rad, out isFair);
          break;
        case Location.Naha:
          sun = new Sun(26, 12.2, 0, 127, 41.3, 0, 135, 0, 0);
          RandomWeather.MakeWeather(DateTime.Now.Millisecond, RandomWeather.Location.Naha, 20, out dbt, out hrt, out rad, out isFair);
          break;
        default:
          sun = new Sun(35, 41.2, 0, 139, 45.9, 0, 135, 0, 0);
          RandomWeather.MakeWeather(DateTime.Now.Millisecond, RandomWeather.Location.Tokyo, 20, out dbt, out hrt, out rad, out isFair);
          break;
      }
      using (StreamWriter sWriter1 = new StreamWriter("TestResult.csv", false, Encoding.GetEncoding("Shift_JIS")))
      using (StreamWriter sWriter2 = new StreamWriter("UnryoDay2.csv", false, Encoding.GetEncoding("Shift_JIS")))
      {
        sWriter1.WriteLine("日付,時刻,乾球温度[C], 絶対湿度[g/kg], 水平面全天日射[W/m2],大気圏外水平面日射[W/m2], sin(太陽高度)[-], 計算成功[-], 大気透過率[-], 雲量");
        sun.Update(dt);
        DateTime sunset = sun.GetSunSetTime();
        DateTime sunrise = sun.GetSunRiseTime();

        for (int i = 0; i < dbt.Length; i++)
        {
          if (i % 24 == 0)
          {
            bool isSame = true;
            bool isF = isFair[i];
            for (int j = 1; j < 24; j++) if (isF != isFair[i + j]) isSame = false;
            int un;
            if (!isSame) un = 5;
            else if (isF) un = 0;
            else un = 10;
            sWriter2.WriteLine(dt.ToShortDateString() + "," + un);
          }

          //日の出・日没時刻更新
          if (sunset.Day != dt.Day)
          {
            sun.Update(dt);
            sunset = sun.GetSunSetTime();
            sunrise = sun.GetSunRiseTime();
          }

          sWriter1.Write(dt.ToShortDateString() + "," + dt.Hour + "," + dbt[i] + "," + hrt[i] + "," + rad[i] + ",");

          //日中ならば正規化//日没日の出から前後30分は誤差大きいためカット
          if ((sunrise.AddMinutes(60) < dt) && (dt < sunset.AddMinutes(-60)))
          {
            sun.Update(dt);
            //大気圏外日射
            double sinH = Math.Sin(sun.Altitude);
            double exr = sun.GetExtraterrestrialRadiation();
            //大気透過率[-]
            bool suc;
            double atmTrans = getATMTrans(sinH, exr, rad[i], out suc);
            //書き出し
            sWriter1.Write((exr * sinH) + "," + Math.Sin(sun.Altitude) + "," + suc.ToString() + "," + atmTrans + ", ");
          }
          else sWriter1.Write("0,0,0,0,");

          int unryo;
          if (isFair[i]) unryo = 0;
          else unryo = 10;
          sWriter1.WriteLine(unryo);

          dt = dt.AddHours(1);
          if (dt.Month == 2 && dt.Day == 29)
          {
            sWriter2.WriteLine(dt.ToShortDateString() + ",0");
            dt = dt.AddDays(1);
          }
        }
      }
    }

    #endregion

    #region サンプルデータ作成処理

    private static void makeSampleData()
    {
      double[] dbt, hrt, rad;
      bool[] isf;
      RandomWeather.MakeWeather(0, RandomWeather.Location.Tokyo, 21, out dbt, out hrt, out rad, out isf);

      DateTime dt = new DateTime(1981, 1, 1, 0, 0, 0);
      StreamWriter swi = null;
      StreamWriter swt = null;
      StreamWriter swx = null;
      int ind = 0;
      while (true)
      {
        if (dt.Year == 2001) break;
        if (dt.Month == 1 && dt.Day == 1 && dt.Hour == 0)
        {
          if (swt != null) swt.Close();
          if (swi != null) swi.Close();
          if (swx != null) swx.Close();
          swt = new StreamWriter("data\\" + dt.Year + "T.CSV", false, Encoding.GetEncoding("Shift_JIS"));
          swi = new StreamWriter("data\\" + dt.Year + "i.CSV", false, Encoding.GetEncoding("Shift_JIS"));
          swx = new StreamWriter("data\\" + dt.Year + "x.CSV", false, Encoding.GetEncoding("Shift_JIS"));
          swt.Write("[℃]"); swi.Write("[MJ/(m2h)]"); swx.Write("[g/kg']");
          for (int i = 0; i < 24; i++)
          {
            swt.Write("," + i); swi.Write("," + i); swx.Write("," + i);
          }
          swt.WriteLine(",Daily Mean"); swi.WriteLine(",Daily Amount"); swx.WriteLine(",Daily Mean");
        }
        swt.Write(dt.ToShortDateString());
        swi.Write(dt.ToShortDateString());
        swx.Write(dt.ToShortDateString());
        double tSum = 0;
        double iSum = 0;
        double xSum = 0;
        for (int i = 0; i < 24; i++)
        {
          swt.Write("," + dbt[ind].ToString("F1"));
          swi.Write("," + (rad[ind] * 0.0036).ToString("F2"));
          swx.Write("," + (hrt[ind]).ToString("F1"));

          tSum += dbt[ind];
          iSum += rad[ind];
          xSum += hrt[ind];
          ind++;
        }
        swt.WriteLine("," + (tSum / 24d).ToString("F1"));
        swi.WriteLine("," + (iSum * 0.0036).ToString("F2"));
        swx.WriteLine("," + (xSum / 24d).ToString("F1"));
        dt = dt.AddDays(1);
      }
      if (swt != null) swt.Close();
      if (swi != null) swi.Close();
      if (swx != null) swx.Close();
    }

    #endregion

  }
}
